%% teleoperation_starter.m
%
% This script simulates a rotational one-degree-of-freedom master-slave
% teleoperation system.  The master and slave are identical devices.  The
% movement of the master is dictated by a pre-recorded trajectory or a
% function of your choosing.  Your job is to write a controller that
% connects the two devices to provide good transparency and stability.
% 
% Written by Katherine J. Kuchenbecker for MEAM 520 at the University of Pennsylvania.
%
% You need to complete this script.  Write your name below and change the
% name of the script to use your PennKey instead of starter.


%% Clean up

clear all
home

% Set student name.
studentName = 'PUT YOUR NAME HERE';


%% Set up the simulation

% Load the trajectory for the master.  You can select from several options.
load masterMovement4

% If you prefer, you can make the master position any function you
% want, either by saving off your own trajectories or by calculating
% them here.  The masterMovement variable should be a column vector
% containing the angle of the master in radians, spaced out at 1000 Hz.
% For example, we can put the master at a constant angle by
% uncommenting the following line.
%masterMovement = 0.2*ones(2000,1);
    
% Set initial angle for the master robot.
masterThetaInitial = masterMovement(1);

% Set initial angle for the slave robot.  It must be global so that
% getSlaveTheta can access it as well.  The slave always starts out
% stationary.
global slaveThetaInitial
slaveThetaInitial = -0.1;

% Set the angle of the obstacle in the slave's workspace, in radians.
% The obstacle extends from this angle in the negative direction.
% It should be between 0 and -2 radians.
% It must be global so getSlaveTheta can access this value as well.
global obstacleAngle;
obstacleAngle = -pi/3;

% Time to wait each servo loop to let the viewer see the animation, in seconds.
% Increase this to slow down plotting, decrease to speed up.
delayTime = 0.010;

% If you don't want to watch the animation, turn the pauses off here.
pause on


%% Set up simulation timing

% Set controller servo rate (number of cycles per second) in Hz.
% Do not change this.
servoRate = 1000;

% Use servo rate to calculate the controller's time step in seconds.
T = 1/servoRate;

% Calculate number of cycles to run the simulation for.
nCycles = length(masterMovement);


%% Select graphics settings

% Locations to graph master and slave, all in meters.
masterX = -.75;
slaveX = .75;
masterSlaveY = 0;
masterSlaveR = 0.5;

% Viewing limits of the main graph, in meters.
minX = -1.5;
maxX = 1.5;
minY = -1.25;
maxY = 1.25;

% Set colors of master, slave, and obstacle for plotting.
masterColor = [0.8 0.0 0.4];
slaveColor = [0.0 0.4 0.8];
obstacleColor = [.3 .8 0];


%% Set up the main figure

% Open figure 1.
figure(1);
clf

% Plot the ground below the master and slave.
fill([minX masterX (masterX+slaveX)/2 slaveX maxX maxX minX],[masterSlaveY-masterSlaveR masterSlaveY masterSlaveY-masterSlaveR masterSlaveY masterSlaveY-masterSlaveR minY minY],[.2 .2 .2],'edgealpha',0)

% Turn hold on to let us put more things on the graph.
hold on

% Plot the obstacle in the slave's workspace.
fill([slaveX (slaveX - 2*masterSlaveR*sin(obstacleAngle)) maxX maxX], [masterSlaveY (masterSlaveY+2*masterSlaveR*cos(obstacleAngle)) (masterSlaveY+2*masterSlaveR*cos(obstacleAngle)) masterSlaveY-masterSlaveR],obstacleColor,'edgealpha',0);

% Put a text label in the obstacle.
text(slaveX+masterSlaveR,masterSlaveY-0.25*masterSlaveR,'Obstacle','horizontalAlignment','center')

% Draw the one-dof master robot as a line.  We keep the handle to this plot
% (hMaster) so that we can move it later. 
hMaster = plot([masterX (masterX-masterSlaveR*sin(masterThetaInitial))], [masterSlaveY (masterSlaveY+masterSlaveR*cos(masterThetaInitial))],'.-','markersize',30,'linewidth',4,'color',masterColor);

% Put a text label below the master.
text(masterX,masterSlaveY-1.25*masterSlaveR,'Master','color',masterColor,'horizontalAlignment','center')

% Draw the one-dof slave robot as a line.  We keep the handle to this plot
% (hSlave) so that we can move it later. 
hSlave = plot([slaveX (slaveX-masterSlaveR*sin(slaveThetaInitial))], [masterSlaveY (masterSlaveY+masterSlaveR*cos(slaveThetaInitial))],'.-','markersize',30,'linewidth',4,'color',slaveColor);

% Put a text label below the slave.
text(slaveX,masterSlaveY-1.25*masterSlaveR,'Slave','color',slaveColor,'horizontalAlignment','center')

% Write the time in the air.
ht = text((slaveX+masterX)/2,masterSlaveY+1.5*masterSlaveR,'t = 0.00 s','horizontalAlignment','center');

% Turn hold off to stop allowing more graphing.
hold off;

% Set the axis limits and other properties of the graph.
axis equal
axis([minX maxX minY maxY])
box on

% Label the axes.
xlabel('X (m)')
ylabel('Y (m)')

% Add a title.
title(['Teleoperation Controller by ' studentName])


%% Do final preparations

% Initialize vectors to hold the time stamp, master and slave angle, and
% master and slave torque for each cycle.
timeHistory = zeros(nCycles,1);
masterThetaHistory = zeros(nCycles,1);
slaveThetaHistory = zeros(nCycles,1);
masterTauHistory = zeros(nCycles,1);
slaveTauHistory = zeros(nCycles,1);

% Declare slave torque as a global variable and set it to zero Nm.
% It must be global so getSlaveTheta can access it as well.
global slaveTau;
slaveTau = 0.00;

%% Run the servo loop
for i = 1:nCycles

    % Calculate the present time.
    t = (i-1)*T;
    
    % Pull the next master position from the movement trajectory.
    masterTheta = masterMovement(i);
    
    % Measure the slave's current position (calculated from its dynamics).
    slaveTheta = getSlaveTheta;
    
    % All of your commands should be below this line of stars.
    % ********************************************************************

    % Calculate the torque to command on the master, in Nm, and store in
    % masterTau.  For now we set it to zero.
    masterTau = 0;
    
    % Calculate the torque to command on the slave, in Nm, and store in
    % slaveTau.  For now we set it to zero.
    slaveTau = 0;
      
    % ********************************************************************
    % All of your commands should be above this line of stars.
    
    % Store data in history vectors for plotting.
    timeHistory(i) = t;
    masterThetaHistory(i) = masterTheta;
    slaveThetaHistory(i) = slaveTheta;
    masterTauHistory(i) = masterTau;
    slaveTauHistory(i) = slaveTau;
    
    % Update graphics periodically.
    if (~mod(i,40))
        % Update the master, slave, and time graphics.
        set(hMaster,'xdata',[masterX (masterX-masterSlaveR*sin(masterTheta))],'ydata',[masterSlaveY (masterSlaveY+masterSlaveR*cos(masterTheta))]);
        set(hSlave,'xdata',[slaveX (slaveX-masterSlaveR*sin(slaveTheta))],'ydata',[masterSlaveY (masterSlaveY+masterSlaveR*cos(slaveTheta))]);
        set(ht,'string',sprintf('t = %.2f s',t))

        % Pause for a short time to let the user see the movement.
        pause(delayTime);
    end
end


%% Graph the resulting data

% Plot the angle data on the upper subplot.
figure(2);
ax(1) = subplot(2,1,1);
plot(timeHistory,masterThetaHistory,'-','color',masterColor)
hold on;
plot(timeHistory,slaveThetaHistory,'-','color',slaveColor)
plot(timeHistory,obstacleAngle*ones(nCycles,1),'--','color',obstacleColor)
hold off;
xlabel('Time (s)')
ylabel('Angle (rad)')
legend('Master','Slave','Obstacle','Location','Best')

% Plot the torque data on the lower subplot.
ax(2) = subplot(2,1,2);
plot(timeHistory,masterTauHistory,'-','color',masterColor)
hold on;
plot(timeHistory,slaveTauHistory,'-','color',slaveColor)
hold off;
xlabel('Time (s)')
ylabel('Commanded Torque (Nm)')
legend('Master','Slave','Location','Best')

% Link their axes so that zooming keeps the x limits matched.
linkaxes(ax,'x')